#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include "nrf_libuarte_async.h"
#include "nrf_drv_clock.h"
#include "nrf_delay.h"
#include "nrf_gpio.h"


/**
 *  Uart pinout configuration
 */
//#define NRF_TX_PIN_NUMBER 22
//#define NRF_RX_PIN_NUMBER 23
#define NRF_TX_PIN_NUMBER 26
#define NRF_RX_PIN_NUMBER 27
/**
 *  Uart instance definition
 */
NRF_LIBUARTE_ASYNC_DEFINE(libuarte, 0, 0, 0, NRF_LIBUARTE_PERIPHERAL_NOT_USED, 255, 3);

/**
 *  Uart event handler function
 */
void uartEventHandler(void * context, nrf_libuarte_async_evt_t * p_evt)
{
    nrf_libuarte_async_t * p_libuarte = (nrf_libuarte_async_t *)context;
    switch (p_evt->type)
    {
        case NRF_LIBUARTE_ASYNC_EVT_ERROR:
            printf("UART ERROR OCCURRED!\n");
            break;

        case NRF_LIBUARTE_ASYNC_EVT_RX_DATA:
            printf("DATA RECIEVED >> %s\n", p_evt->data.rxtx.p_data);
            nrf_libuarte_async_rx_free(p_libuarte, p_evt->data.rxtx.p_data, p_evt->data.rxtx.length);
            break;

        default:
            break;
    }
}

/**
 *  Uart initialisation function
 */
void uartInit()
{
    ret_code_t err_code;
    nrf_libuarte_async_config_t config = {
        .tx_pin     = NRF_TX_PIN_NUMBER,
        .rx_pin     = NRF_RX_PIN_NUMBER,
        .baudrate   = NRF_UARTE_BAUDRATE_115200,
        .parity     = NRF_UARTE_PARITY_EXCLUDED,
        .hwfc       = NRF_UARTE_HWFC_DISABLED,
        .timeout_us = 100,
        .int_prio   = APP_IRQ_PRIORITY_LOW
    };
    err_code = nrf_libuarte_async_init(&libuarte, &config, uartEventHandler, (void *)&libuarte);
    APP_ERROR_CHECK(err_code);
    nrf_libuarte_async_enable(&libuarte);
}

/**
 *  Uart write function
 */
bool uartWrite(uint8_t *data, size_t data_lenght)
{
    ret_code_t err_code;
    err_code = nrf_libuarte_async_tx(&libuarte, data, data_lenght);
    APP_ERROR_CHECK(err_code);
    return err_code == NRF_SUCCESS;
}

int main(void)
{

//   nrf_gpio_cfg_output(15);
   // nrf_gpio_cfg_output(3);
   //nrf_gpio_pin_set(15);
   // nrf_gpio_pin_set(3);
//   nrf_delay_ms(2000);


   //nrf_gpio_pin_clear(15);
   // nrf_gpio_pin_clear(3);
//   nrf_delay_ms(2000);

    // clock configuration
    ret_code_t ret = nrf_drv_clock_init();
    APP_ERROR_CHECK(ret);
    nrf_drv_clock_lfclk_request(NULL);

    // initialize the uart connection
    uartInit();

    uint8_t b[] = "AT\r\n";

    uartWrite(b, strlen(b));
//
       nrf_delay_ms(2000);
//
    uint8_t a[] = "AT+CGNSPWR=1\r\n";
//
    uartWrite(a, strlen(a));
        nrf_delay_ms(2000);
//
//
     uint8_t c[] = "AT+CGNSINF\r\n";
        uartWrite(c, strlen(c));
//
        nrf_delay_ms(2000);

    // for(int i = 0; i<10;i++){
    //     uartWrite(c, strlen(c));
    //     nrf_delay_ms(2000);

    // }
         while(true){
         uartWrite(c, strlen(c));
         nrf_delay_ms(4000);

         }


    return 1;
}